<?php
require __DIR__ . "/vendor/autoload.php";
require __DIR__ . "/vendor/zeedhi/framework/bootstrap.php";
require __DIR__ . "/vendor/teknisa/libraries/scripts/bootstrap.php";  

use Doctrine\DBAL\Types\Type;

/**
 * @todo remove setforms2 entities and classes
 */

$instanceManager = \Zeedhi\Framework\DependencyInjection\InstanceManager::getInstance();
$routes = array();

$mapping = getMapping();

loadParameters($instanceManager); //parameters must be loaded before acl and log
configAcl($instanceManager, $routes);
configLog($instanceManager, $routes);
configPreferences($instanceManager, $routes);
loadConfigFiles($instanceManager); //config files must be loaded after acl and log, because they overwrite LoggerInfoProvider as an abstract class
loadRoutesAndServices($mapping, $instanceManager, $routes);

$instanceManager->compile();

useSetform2Classes($instanceManager);

function getMapping() {
	$mapping = file_get_contents(__DIR__ . "/config/servicesMapping.json");
	return json_decode($mapping, true);
}

function loadRoutesAndServices($mapping, $instanceManager, &$routes) {
	foreach ($mapping as $path) {
		$ext = getFilePathExtension($path);
		if($ext === 'xml') {
			$instanceManager->loadFromFile(__DIR__ . $path);
		}
		if($ext === 'json') {
			$routes[] = __DIR__ . $path;
		}
	}
	$routes[] = __DIR__ . "/../../../backend/routes.json";
	$routes[] = __DIR__ . "/vendor/teknisa/libraries/libRoutes.json";
	$instanceManager->registerService("routeParser", new \Zeedhi\Framework\Routing\Parsers\JSONFormat($routes));
}

function loadParameters($instanceManager) {
	$parameters = __DIR__ . "/../../../backend/config/parameters.xml"; // prioritize product parameters
	if (realpath($parameters)) {
		$instanceManager->loadFromFile($parameters);
	}
}

function loadConfigFiles($instanceManager) {
	$db 	= __DIR__ . "/../../../backend/config/db.xml"; // prioritize product database config
	$report = __DIR__ . "/config/report.xml"; // prioritize microservice report config
	$general = __DIR__ . "/../../../backend/config/general.xml"; // prioritize product general services
	$environment = __DIR__ . "/config/environment.xml"; // prioritize product environment services
	$fileUploader = __DIR__ . "/config/fileUploader.xml";
	
	if (!realpath($db)) {
		$db = __DIR__."/config/db.xml";
	}
	if (!realpath($report)) {
		$report = __DIR__ . "/../../../backend/config/report.xml";
	}
	if (realpath($general)) {
		$instanceManager->loadFromFile($general);
	}
	if (realpath($environment)) {
		$instanceManager->loadFromFile($environment);
	}
	if (realpath($fileUploader)) {
		$instanceManager->loadFromFile($fileUploader);
	}
	
	$instanceManager->loadFromFile($db);
	$instanceManager->loadFromFile($report);
}

function configAcl($instanceManager, &$routes) {
	$container = $instanceManager->getContainer();
	$aclParam = "use_zhu_acl";
	if ($container->hasParameter($aclParam) && $container->getParameter($aclParam)) {
		$acl = __DIR__ . "/config/acl.xml";
		if(realpath($acl)) {
			$instanceManager->loadFromFile($acl);
		}
		require __DIR__ . "/vendor/zeedhi/zhuaclbackend/service/bootstrap.php";
		$routes[] = __DIR__ . "/vendor/zeedhi/zhuaclbackend/config/routes.json";
	}
}

function configLog($instanceManager, &$routes) {
	$container = $instanceManager->getContainer();
	$logParam = "use_zhu_log";
	if ($container->hasParameter($logParam) && $container->getParameter($logParam)) {
		$zhuDateFormat = $container->getParameter('zhuLogDateFormat');
		$ignoredRoutes = $container->hasParameter('logIgnoredRoutes') ? $container->getParameter('logIgnoredRoutes') : [];
		$log = __DIR__.'/../../acl/backend/config/log.xml';
		if(realpath($log)) {
			$instanceManager->loadFromFile($log);
		}
		
		$libLog = __DIR__.'/vendor/teknisa/libraries/zhuLog.xml';
		if(realpath($libLog)){
			$instanceManager->loadFromFile($libLog);
		}
		
		require __DIR__ . "/vendor/zeedhi/zhulog/config/bootstrap.php";
		$routes[] = __DIR__ . "/vendor/zeedhi/zhulog/config/routes.json";
		
		$container->setParameter('logIgnoredRoutes', $ignoredRoutes);
		$container->setParameter('zhuLogDateFormat', $zhuDateFormat);
	}
}

function configPreferences($instanceManager, &$routes) {
	$routes[] = __DIR__ . "/vendor/zeedhi/preferences/routes.json";
	$instanceManager->loadFromFile(__DIR__.'/vendor/zeedhi/preferences/services.xml');
}

function useSetform2Classes($instanceManager) {
	Type::overrideType(Type::DATE, 'Setforms2\Shared\ORM\DateType');
	Type::overrideType(Type::DATETIME, 'Setforms2\Shared\ORM\DateTimeType');

	$entity_manager = $instanceManager->getService('entityManager');
	\Setforms2\Shared\Registry::store("EntityManager", $entity_manager);
	\Setforms2\Shared\Registry::store("EntityManagerConnection", $entity_manager->getConnection());
}

function getFilePathExtension($path) {
	return substr(strrchr($path, '.'), 1);
}